/* byteswap.h
 * Copyright (C) 2002-2003 OpenQuicktime Team
 *
 * This file is part of OpenQuicktime, a free QuickTime library.
 *
 * OpenQuicktime is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation.
 *
 * OpenQuicktime is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $ID:$
 */
 

/*!
	@header OpenQuicktime Byteswap
	This header provides byteswapping functions for systems without byteswap.h
*/

#ifndef OQT_BYTESWAP_H__
#define OQT_BYTESWAP_H__

#ifndef OPENQUICKTIME_CONFIG_H__
#include "openquicktime/config.h"
#endif

#ifndef OPENQUICKTIME_STRUCTS_H__
#include "openquicktime/structs.h"
#endif

#ifdef HAVE_BYTESWAP_H
#include <byteswap.h>
#else
	/* Inline byteswap functions for systems that don't 
	   have byteswap.h (everything apart from Linux)
	   Borrowed from XTL by Jose' Orlando Pereira       */
	
	static inline oqt_uint16_t bswap_16(oqt_uint16_t b) {
		return ((b&0xff00)>>8)|((b&0x00ff)>>8);
	}
	
	static inline oqt_uint32_t bswap_32(oqt_uint32_t b) {
		return ((b&0xff000000)>>24)|((b&0x00ff0000)>>8)|
			((b&0x0000ff00)<<8)|((b&0x000000ff)<<24);
	}
	
	static inline oqt_uint64_t bswap_64(oqt_uint64_t b) { 
		return ( ((b&0xff00000000000000LL) >> 56) |
			 ((b&0x00ff000000000000LL) >> 40) |
			 ((b&0x0000ff0000000000LL) >> 24) |
			 ((b&0x000000ff00000000LL) >> 8) |
			 ((b&0x00000000ff000000LL) << 8) |
			 ((b&0x0000000000ff0000LL) << 24) |
			 ((b&0x000000000000ff00LL) << 40) |
			 ((b&0x00000000000000ffLL) << 56) );
	}
#endif


// be2me ... BigEndian to MachineEndian
// le2me ... LittleEndian to MachineEndian
#ifdef WORDS_BIGENDIAN
#define be2me_16(x) (x)
#define be2me_32(x) (x)
#define be2me_64(x) (x)
#define le2me_16(x) bswap_16(x)
#define le2me_32(x) bswap_32(x)
#define le2me_64(x) bswap_64(x)
#else
#define be2me_16(x) bswap_16(x)
#define be2me_32(x) bswap_32(x)
#define be2me_64(x) bswap_64(x)
#define le2me_16(x) (x)
#define le2me_32(x) (x)
#define le2me_64(x) (x)
#endif

#endif
