/* structs.h
 *
 * Data Structure Definitions
 * Copyright (C) 2001 QT4Linux and OpenQuicktime Teams
 *
 * This file is part of OpenQuicktime, a free QuickTime library.
 *
 * Based on QT4Linux by Adam Williams.
 *
 * OpenQuicktime is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation.
 *
 * OpenQuicktime is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $Id: structs.h,v 1.36 2003/04/07 20:58:46 shitowax Exp $
 */

#ifndef OPENQUICKTIME_STRUCTS_H__
#define OPENQUICKTIME_STRUCTS_H__
 
/* ================================= structures */

#define OQT_ATOM_HEAD_LENGTH	8
#define OQT_MAX_TRACKS			1024

#include <stdio.h>
#include <sys/types.h>

#ifndef WIN32
 #ifndef __MACOS__
  #ifndef OPENQUICKTIME_CONFIG_H__
   #include "openquicktime/config.h"
  #endif
  #define OPENQUICKTIMELIB_API
  #define OQT_STDCALL

 #else /* __MACOS__ */
 #ifndef __MACOS__
  #include "macconfig.h"
  #define OPENQUICKTIMELIB_API
  #define OQT_STDCALL
 #endif
 #endif /* __MACOS__ */

#else /* WIN32 */
 #include "winconfig.h"
 #ifdef STATICLIB
  #undef OPENQUICKTIMELIB_EXPORTS
  #define OPENQUICKTIMELIB_API
 #else
  #ifdef OPENQUICKTIMELIB_EXPORTS
   #define OPENQUICKTIMELIB_API __declspec(dllexport)
  #else
   #define OPENQUICKTIMELIB_API __declspec(dllimport)
  #endif
 #endif 
#endif /* WIN32 */

#if defined(WIN32) && defined(STATICLIB)
#define OQT_STDCALL _stdcall
#else
#define OQT_STDCALL
#endif

/* OpenQuicktime's own integer types */
typedef __s8	oqt_int8_t;
typedef __s16	oqt_int16_t;
typedef __s32	oqt_int32_t;
typedef __s64	oqt_int64_t;
typedef __u8	oqt_uint8_t;
typedef __u16	oqt_uint16_t;
typedef __u32	oqt_uint32_t;
typedef __u64	oqt_uint64_t;
typedef float	oqt_float_t;


typedef struct
{
	oqt_int64_t start;      /* byte start in file */
	oqt_int64_t end;        /* byte endpoint in file */
	oqt_int64_t size;       /* byte size for writing */
	oqt_int32_t use_64;     /* Use 64 bit header */
    oqt_int8_t  type[4];
} oqt_atom_t;

typedef struct
{
	oqt_float_t values[9];
} oqt_matrix_t;


typedef struct
{
	oqt_int32_t  version;
	oqt_int32_t  flags;
	oqt_uint32_t creation_time;
	oqt_uint32_t modification_time;
	oqt_int32_t  track_id;
	oqt_int32_t  reserved1;
	oqt_int32_t  duration;
	oqt_int8_t   reserved2[8];
	oqt_int32_t  layer;
	oqt_int32_t  alternate_group;
	oqt_float_t    volume;
	oqt_int32_t  reserved3;
	oqt_matrix_t matrix;
	oqt_float_t track_width;
	oqt_float_t track_height;
} oqt_tkhd_t;


typedef struct
{
	oqt_int32_t seed;
	oqt_int32_t flags;
	oqt_int32_t size;

	oqt_int16_t *alpha;
	oqt_int16_t *red;
	oqt_int16_t *green;
	oqt_int16_t *blue;
} oqt_ctab_t;



/* ===================== sample table ======================== */



/* sample description */

typedef struct
{
	oqt_int32_t motion_jpeg_quantization_table;
} oqt_mjqt_t;


typedef struct
{
	oqt_int32_t motion_jpeg_huffman_table;
} oqt_mjht_t;


#define		OQT_STSDWAVE_PCM		(0x00)
//#define		OQT_STSDWAVE_MPEG1	(0x50)
#define		OQT_STSDWAVE_MPEG	(0x55)

typedef struct
{
	char	fmtType[4];
	int	 	little_endian;

	oqt_int16_t	wFormatTag;			// 0x55 for MP3
	oqt_int16_t	nChannels;			// Number of channels in the wave: 1 for mono, 2 for stereo.
	oqt_int32_t	nSamplesPerSec;		// Sampling frequency (in Hz) of the wave file: 32000, 44100, or 48000 etc.
	oqt_int32_t	nAvgBytesPerSec;	// Average data rate
	oqt_int16_t	nBlockAlign;		// For streams in which the frame length varies, set to 1.
									// Layer 1: nBlockAlign = 4*(int)(12*BitRate/SamplingFreq)
									// Layers 2 and 3: nBlockAlign = (int)(144*BitRate/SamplingFreq)
	oqt_int16_t	wBitsPerSample;		// Not used; set to zero. (or 16?)
	oqt_int16_t	cbSize;				// The size in bytes of the extended data after the WAVEFORMATEX structure.
									// For MPEG1WAVEFORMAT this is 22 (0x0016).

	/* The MPEG Audio Extension
		- thanks to Adrian Bourke for help with this
	*/
    oqt_int16_t	wID;
	oqt_int32_t	fdwFlags;
	oqt_int16_t	nBlockSize;
	oqt_int16_t	nFramesPerBlock;
	oqt_int16_t	nCodecDelay;
	
	/* Other formats are shoved in buffer */
	oqt_int32_t	extra_len;
	oqt_int8_t	*extra;
	
} oqt_stsdwave_t;


/* MP4 Elementary Stream Descriptor */
typedef struct
{
	int			version;
	oqt_int32_t	flags;
	int			decoderConfigLen;
	int			objectTypeId;
	int			streamType;
	int			bufferSizeDB;
	oqt_int32_t	maxBitrate;
	oqt_int32_t	avgBitrate;
	oqt_uint8_t*	decoderConfig;
} oqt_esds_t;

typedef struct
{
	char format[4];
	oqt_int8_t reserved[6];
	oqt_int32_t data_reference;

/* common to audio and video */
	oqt_int32_t version;
	oqt_int32_t revision;
	char vendor[4];

/* video description */
	oqt_int32_t temporal_quality;
	oqt_int32_t spatial_quality;
	oqt_int32_t width;
	oqt_int32_t height;
	oqt_float_t dpi_horizontal;
	oqt_float_t dpi_vertical;
	oqt_int64_t data_size;
	oqt_int32_t frames_per_sample;
	char compressor_name[32];
	oqt_int32_t depth;
	oqt_int32_t ctab_id;
	oqt_ctab_t ctab;
	oqt_float_t gamma;
	oqt_int32_t fields;    /* 0, 1, or 2 */
	oqt_int32_t field_dominance;   /* 0 - unknown     1 - top first     2 - bottom first */
	oqt_mjqt_t mjqt;
	oqt_mjht_t mjht;

	oqt_int32_t strf_len;
	oqt_uint8_t *strf;	/* 'strf' chunk from AVI format */


/* audio description */
	oqt_int32_t channels;
	oqt_int32_t sample_size;
	oqt_int32_t compression_id;
	oqt_int32_t packet_size;
	oqt_float_t sample_rate;

/* audio description V1 */
	oqt_uint32_t samplesPerPacket;
	oqt_uint32_t bytesPerPacket;
	oqt_uint32_t bytesPerFrame;
	oqt_uint32_t bytesPerSample;
	
/* Wave Extension */
	oqt_stsdwave_t wave;

/* MP4 ESDS */
	oqt_esds_t esds;

/* identical to first table entry with only data_reference changed */
	int same_but_external;

} oqt_stsd_table_t;


typedef struct
{
	oqt_int32_t version;
	oqt_int32_t flags;
	oqt_int32_t total_entries;
	oqt_stsd_table_t *table;
} oqt_stsd_t;


/* time to sample */
typedef struct
{
	oqt_int32_t sample_count;
	oqt_int32_t sample_duration;
} oqt_stts_table_t;

typedef struct
{
	oqt_int32_t version;
	oqt_int32_t flags;
	oqt_int32_t total_entries;
	oqt_stts_table_t *table;
} oqt_stts_t;


/* sync sample */
typedef struct
{
	oqt_int32_t sample;
} oqt_stss_table_t;

typedef struct
{
	oqt_int32_t version;
	oqt_int32_t flags;
	oqt_int32_t total_entries;
	oqt_int32_t entries_allocated;
	oqt_stss_table_t *table;
} oqt_stss_t;


/* sample to chunk */
typedef struct
{
	oqt_int32_t chunk;
	oqt_int32_t samples;
	oqt_int32_t id;
} oqt_stsc_table_t;

typedef struct
{
	oqt_int32_t version;
	oqt_int32_t flags;
	oqt_int32_t total_entries;
	
	oqt_int32_t entries_allocated;
	oqt_stsc_table_t *table;
} oqt_stsc_t;


/* sample size */
typedef struct
{
	oqt_int64_t size;
} oqt_stsz_table_t;

typedef struct
{
	oqt_int32_t version;
	oqt_int32_t flags;
	oqt_int64_t sample_size;
	oqt_int32_t total_entries;

	oqt_int32_t entries_allocated;    /* used by the library for allocating a table */
	oqt_stsz_table_t *table;
} oqt_stsz_t;


/* chunk offset */
typedef struct
{
	oqt_int64_t offset;
} oqt_stco_table_t;

typedef struct
{
	oqt_int32_t version;
	oqt_int32_t flags;
	oqt_int32_t total_entries;
	
	oqt_int32_t entries_allocated;    /* used by the library for allocating a table */
	oqt_stco_table_t *table;
} oqt_stco_t;


/* sample table */
typedef struct
{
	oqt_int32_t version;
	oqt_int32_t flags;
	oqt_stsd_t stsd;
	oqt_stts_t stts;
	oqt_stss_t stss;
	oqt_stsc_t stsc;
	oqt_stsz_t stsz;
	oqt_stco_t stco;
} oqt_stbl_t;


/* data reference */
typedef struct
{
	oqt_int64_t size;
	char type[4];
	oqt_int32_t version;
	oqt_int32_t flags;
	char *data_reference;
} oqt_dref_table_t;

typedef struct
{
	oqt_int32_t version;
	oqt_int32_t flags;
	oqt_int32_t total_entries;
	oqt_dref_table_t *table;
} oqt_dref_t;


/* data information */
typedef struct
{
	oqt_dref_t dref;
} oqt_dinf_t;


/* video media header */
typedef struct
{
	oqt_int32_t version;
	oqt_int32_t flags;
	oqt_int32_t graphics_mode;
	oqt_int32_t opcolor[3];
} oqt_vmhd_t;


/* sound media header */
typedef struct
{
	oqt_int32_t version;
	oqt_int32_t flags;
	oqt_int32_t balance;
	oqt_int32_t reserved;
} oqt_smhd_t;


/* handler reference */
typedef struct
{
	oqt_int32_t version;
	oqt_int32_t flags;
	char component_type[4];
	char component_subtype[4];
	char component_manufacturer[4];
	//oqt_int32_t component_manufacturer;
	oqt_int32_t component_flags;
	oqt_int32_t component_flag_mask;
	char component_name[256];
} oqt_hdlr_t;


/* media information */
typedef struct
{
	oqt_int32_t is_video;
	oqt_int32_t is_audio;
	oqt_vmhd_t vmhd;
	oqt_smhd_t smhd;
	oqt_stbl_t stbl;
	oqt_hdlr_t hdlr;
	oqt_dinf_t dinf;
} oqt_minf_t;



/* media header */
typedef struct
{
	oqt_int32_t version;
	oqt_int32_t flags;
	oqt_uint32_t creation_time;
	oqt_uint32_t modification_time;
	oqt_int32_t time_scale;
	oqt_int32_t duration;
	oqt_int32_t language;
	oqt_int32_t quality;
} oqt_mdhd_t;


/* media */
typedef struct
{
	oqt_mdhd_t mdhd;
	oqt_minf_t minf;
	oqt_hdlr_t hdlr;
} oqt_mdia_t;


/* edit list */
typedef struct
{
	oqt_int32_t duration;
	oqt_int32_t time;
	oqt_float_t rate;
} oqt_elst_table_t;

typedef struct
{
	oqt_int32_t version;
	oqt_int32_t flags;
	oqt_int32_t total_entries;

	oqt_elst_table_t *table;
} oqt_elst_t;

typedef struct
{
	oqt_elst_t elst;
} oqt_edts_t;


typedef struct
{
	oqt_tkhd_t tkhd;
	oqt_mdia_t mdia;
	oqt_edts_t edts;
} oqt_trak_t;


typedef struct
{
	oqt_int32_t version;
	oqt_int32_t flags;
	oqt_uint32_t creation_time;
	oqt_uint32_t modification_time;
	oqt_int32_t time_scale;
	oqt_int32_t duration;
	oqt_float_t preferred_rate;
	oqt_float_t preferred_volume;
	char reserved[10];
	oqt_matrix_t matrix;
	oqt_int32_t preview_time;
	oqt_int32_t preview_duration;
	oqt_int32_t poster_time;
	oqt_int32_t selection_time;
	oqt_int32_t selection_duration;
	oqt_int32_t current_time;
	oqt_int32_t next_track_id;
} oqt_mvhd_t;

typedef struct
{
	oqt_int32_t version;
	oqt_uint32_t flags;
	oqt_int32_t audioProfileId;
	oqt_int32_t videoProfileId;
} oqt_iods_t;



#define OQT_INFO_COPYRIGHT 		"cpy"
#define OQT_INFO_NAME 			"nam"
#define OQT_INFO_INFORMATION 	"inf"
#define OQT_INFO_ARTIST 		"ART"
#define OQT_INFO_ALBUMN 		"alb"
#define OQT_INFO_ENCODED_BY 	"enc"
#define OQT_INFO_TRACK 			"trk"
#define OQT_INFO_CREATION_DATE 	"day"
#define OQT_INFO_COMMENT 		"cmt"
#define OQT_INFO_AUTHOR 		"aut"
#define OQT_INFO_COMPOSER 		"com"
#define OQT_INFO_DESCRIPTION 	"des"
#define OQT_INFO_DIRECTOR 		"dir"
#define OQT_INFO_DISCLAIMER 	"dis"
#define OQT_INFO_GENRE 			"gen"
#define OQT_INFO_HOST_COMPUTER 	"hst"
#define OQT_INFO_MAKE 			"mak"
#define OQT_INFO_MODEL 			"mod"
#define OQT_INFO_ORIG_ARTIST 	"ope"
#define OQT_INFO_ORIG_FORMAT 	"fmt"
#define OQT_INFO_ORIG_SOURCE 	"src"
#define OQT_INFO_PERFORMERS 	"prf"
#define OQT_INFO_PRODUCER 		"prd"
#define OQT_INFO_PRODUCT 		"PRD"
#define OQT_INFO_SOFTWARE 		"swr"
#define OQT_INFO_SPECL_PLAY_REQ "req"
#define OQT_INFO_WARNING 		"wrn"
#define OQT_INFO_WRITER 		"wrt"
#define OQT_INFO_URL_LINK 		"url"
#define OQT_INFO_EDIT_DATE_1 	"ed1"
#define OQT_INFO_EDIT_DATE_2 	"ed2"
#define OQT_INFO_EDIT_DATE_3 	"ed3"
#define OQT_INFO_EDIT_DATE_4 	"ed4"
#define OQT_INFO_EDIT_DATE_5 	"ed5"
#define OQT_INFO_EDIT_DATE_6 	"ed6"
#define OQT_INFO_EDIT_DATE_7 	"ed7"
#define OQT_INFO_EDIT_DATE_8 	"ed8"
#define OQT_INFO_EDIT_DATE_9 	"ed9"

typedef struct
{
	char	code[4];
	char*	value;
	int		value_len;
} oqt_udta_t;


typedef struct
{
	int total_tracks;
	int udta_count;

	oqt_atom_t	atom;
	oqt_mvhd_t	mvhd;
	oqt_trak_t	*trak[OQT_MAX_TRACKS];
	oqt_udta_t	*udta;
	//oqt_ctab_t	ctab;
	oqt_iods_t	iods;

	oqt_int16_t	window_x;
	oqt_int16_t	window_y;
} oqt_moov_t;


typedef struct
{
	oqt_atom_t atom;
} oqt_mdat_t;



struct oqt_audio_codec_struct;
struct oqt_video_codec_struct;


/* table of pointers to every track */
typedef struct oqt_audio_map_struct
{
	oqt_trak_t *track; /* real quicktime track corresponding to this table */
	oqt_int64_t current_sample;   /* current sample in output file */
	oqt_int32_t current_chunk;      /* current chunk in output file */
	
	// Buffers for the chunk being processed
	oqt_uint8_t* decoded_buf_ptr;	/* Buffer containing raw audio samples */
	int decoded_buf_size;	/* Total length of the work buffer */
	int decoded_buf_used;		/* Length of data currently in work buffer */
	
	oqt_uint8_t* encoded_buf_ptr;	/* Buffer containing encoded audio (to/from file) */
	int encoded_buf_size;		/* Length of the current read buffer */
	int encoded_buf_used;		/* Length of data currently in read buffer */
	
	int encoder_preped;	/* Set to true when encoder is ready to go */
	int samples_per_chunk;	/* Number of samples for each chunk when encoding */
	
	void *codec_private; /* Pointer to structure used internally by codecs */
	struct oqt_audio_codec_struct *codec;
} oqt_audio_map_t;



typedef struct oqt_video_map_struct
{
	oqt_trak_t *track;
	oqt_int64_t current_frame;	/* current frame in output file */
	oqt_int32_t current_chunk;	/* current chunk in output file */

	void *codec_private; /* Pointer to structure used internally by codecs */
	struct oqt_video_codec_struct *codec;
} oqt_video_map_t;



#define OQT_MODE_READ		0x01
#define OQT_MODE_WRITE		0x02
#define OQT_MODE_READWRITE	0x03


/* file descriptor passed to all routines */
typedef struct oqt_struct
{
	
	/* Memory pointer to stream information used by io_*.c */
	void *stream;
	
	/* Textual description of the stream type (File/Memory) */
	char *stream_type;
	
	/* Stream reference (URL, filepath, IP address) */
	char *stream_reference;
	
	/* Is file Read/Write ? */
	int stream_mode;

	/* Current position in file - only use within io.c
	   Please access using oqt_get_position(), oqt_set_position() */
	oqt_int64_t file_position;

	/* Parameters to handle compressed atoms */
	oqt_int64_t decompressed_buffer_size;
	char *decompressed_buffer;
	oqt_int64_t decompressed_position;
	
	
	/* Read ahead buffer */
	oqt_int64_t preload_size;      /* Enables preload when nonzero. */
	char *preload_buffer;
	oqt_int64_t preload_start;     /* Start of preload_buffer in file */
	oqt_int64_t preload_end;       /* End of preload buffer in file */
	oqt_int64_t preload_ptr;       /* Offset of preload_start in preload_buffer */
	
	
	/* I/O handling callbacks on stream */
	int (*oqt_read) (struct oqt_struct *file, char *data, oqt_int64_t size);
	int (*oqt_fseek) (struct oqt_struct *file, oqt_int64_t offset);
	oqt_int64_t (*oqt_ftell) (struct oqt_struct *file);
	oqt_int64_t (*oqt_flen) (struct oqt_struct *file);
	int (*oqt_write) (struct oqt_struct *file, char *data, oqt_int64_t size);
	int (*oqt_close) (struct oqt_struct *file);

	
	
	
	/* And different kind of plugin as well ;) */
	int (*oqt_init_video_map)(struct oqt_struct *file, int track, oqt_trak_t *trak);
	int (*oqt_init_audio_map)(struct oqt_struct *file, int track, oqt_trak_t *trak);
	int (*oqt_delete_video_map)(struct oqt_struct *file, int track);
	int (*oqt_delete_audio_map)(struct oqt_struct *file, int track);
	
	/* Roots of the atom tree */
	oqt_mdat_t mdat;
	oqt_moov_t moov;

	
	/* mapping of audio tracks to movie tracks */
	int total_atracks;
	oqt_audio_map_t *atracks;
	
	/* mapping of video tracks to movie tracks */
	int total_vtracks;
	oqt_video_map_t *vtracks;

	/* MP4 handling */
	int use_mp4;

	/* external resources */
	int ext_count;
	struct oqt_struct **ext_files;

	/* File to be used in next data access.
	   Set by oqt_set_{vide,audi}o_position. */
	struct oqt_struct *data_file;

} oqt_t;


// Any codec type less than 1 is invalid and won't be added
// to the registry....
#define OQT_CODEC_TYPE_NONE		0
#define OQT_CODEC_TYPE_VIDEO	1
#define OQT_CODEC_TYPE_AUDIO	2
// Only loads other codecs - does nothing useful itself.
#define OQT_CODEC_TYPE_LOADER	3



#define OQT_NORMAL_MERIT 100

typedef struct oqt_codec_info_struct {
	char fourcc[4];
	int type;
	/* OQT_CODEC_TYPE_NONE, (oqt_codec_info_struct)
	   OQT_CODEC_TYPE_VIDEO, (oqt_video_codec_struct)
	   OQT_CODEC_TYPE_AUDIO, (oqt_audio_codec_struct) 
	   OQT_CODEC_TYPE_LOADER, (oqt_codec_info_struct) */
	
	const char *name;
	const char *description;	// Not required
	const char *url;			// URL for *Module* not codec
								// and only set for code outside main source tree
	const char *version;
	const char *copyright;
	const char *license;
	const char *module_author;
	const char *codec_author;
	
	int interface_age;
	char has_children;
	void *plugin_handle;
	struct oqt_codec_info_struct *parent;
	int merit;
	
} oqt_codec_info_t;



/* OpenQuicktime Codec Parameter Types */
#define OQT_PARAMETER_UNKNOWN       	-1
#define OQT_PARAMETER_STRING         	0
#define OQT_PARAMETER_BOOLEAN         	1
#define OQT_PARAMETER_INTEGER         	4
#define OQT_PARAMETER_UNSIGNED_INTEGER	5
#define OQT_PARAMETER_DOUBLE          	8

/* Maximum length for value data passed 
   in and out of callbacks - strings etc... */
#define OQT_PARAMETER_MAX_VALUE_LEN		128


typedef struct
{
	char* name;
	char  type;			// See above
	char* desc;
/* Still to be done
   Function for validating set_value method ?
	void* min_val;
	void* max_val;
	void* default_val;
*/
} oqt_codec_param_t;

typedef struct oqt_video_codec_struct
{
	oqt_codec_info_t info;
	
	int(*init_codec)(oqt_t *file, int track);
	int(*delete_codec)(oqt_t *file, int track);
	const oqt_codec_param_t* (*list_params)(int *o_numparams);
	
	int(*decode)(oqt_t *file,
		   int track,
		   oqt_uint32_t inputsize,
		   oqt_uint8_t *input,
		   oqt_uint8_t **output,
		   int color_model);	/* Colour model of output data */
	
	int(*encode)(oqt_t *file,
		   int track,
		   oqt_uint8_t **input,
		   oqt_uint32_t out_size,  /* Size of output buffer (in bytes) */
		   oqt_uint8_t *output,
		   int *IsAKeyFrame,	 /* Set to 1 if encoded frame is a keyframe */
		   int color_model);	 /* Colour model of input data */
	
	
	/* Colour model routines */
	int (*reads_colormodel)(oqt_t *file,
			int track,  		      
			int color_model);
	
	int (*writes_colormodel)(oqt_t *file, 
			int track,  		      
			int color_model);
			
	/* The colour model to be used for conversion.
	 - if the colour model wanted is not supported by codec */
	int (*preferred_colormodel)(oqt_t *file,
			int track);
	
	
	int (*set_param)(oqt_t *file, int track,
		   const char* param, const void* data);
	int (*get_param)(oqt_t *file, int track,
		   const char* param, void* data);
	
} oqt_video_codec_t;




typedef struct oqt_audio_codec_struct
{
  oqt_codec_info_t info;

  int(*init_codec)(oqt_t *file, int track);
  int(*delete_codec)(oqt_t *file, int track);
  const oqt_codec_param_t* (*list_params)(int *o_numparams);

	
  // Returns the number of bytes put in the output buffer //
  int(*decode)(oqt_t *file,
	       int track,
	       oqt_uint32_t in_size, /* in bytes */
	       oqt_uint8_t  *input,
	       oqt_uint32_t out_size, /* in bytes */
	       oqt_uint8_t  *output);

  /* Call just before doing any encoding 
     - returns the number of samples to put in each chunk
     - returns 0 to use default chunk size
     - returns -1 if audio format is unsupported/error
  */
  int(*prepare_encoder)(oqt_t *file, int track);
  
  /* Converts a number of frames of audio to its length in bytes.
     Audio codecs are only required to give this if it is possible to
     calculate - more complex codecs can rely on the STSZ table. */
  int(*frames_to_bytes)(oqt_t *file, int track, int frames);
  
  int(*encode)(oqt_t *file,
	       int track,
	       oqt_uint32_t num_samples,  /* Number of samples in the L (or R) input buffer */
	       oqt_uint8_t *input,
	       oqt_uint32_t out_size,  /* Size of output buffer (in bytes) */
	       oqt_uint8_t *output);

  int (*set_param)(oqt_t *file, int track,
		   const char* param, const void* data);
  int (*get_param)(oqt_t *file, int track,
		   const char* param, void* data);
 
} oqt_audio_codec_t;



#endif
